import sharp from 'sharp';
import axios from 'axios';
import path from 'path';
import { mkdir } from 'fs/promises';
import { existsSync } from 'fs';
import { TSPLConverter } from './tsplConverter.js';

export class ImageProcessor {
  constructor() {
    this.tsplConverter = new TSPLConverter();
  }

  async downloadImage(url) {
    try {
      const response = await axios.get(url, { responseType: 'arraybuffer' });
      return Buffer.from(response.data);
    } catch (error) {
      throw new Error(`下载图片失败: ${error.message}`);
    }
  }

  async ensureDirectoryExists(dir) {
    if (!existsSync(dir)) {
      await mkdir(dir, { recursive: true });
    }
  }

  calculateOtsuThreshold(histogram, total) {
    let sum = 0;
    for (let i = 0; i < 256; i++) {
      sum += i * histogram[i];
    }

    let sumB = 0;
    let wB = 0;
    let wF = 0;
    let maxVariance = 0;
    let threshold = 0;

    for (let t = 0; t < 256; t++) {
      wB += histogram[t];
      if (wB === 0) continue;

      wF = total - wB;
      if (wF === 0) break;

      sumB += t * histogram[t];
      const mB = sumB / wB;
      const mF = (sum - sumB) / wF;

      const variance = wB * wF * Math.pow(mB - mF, 2);
      if (variance > maxVariance) {
        maxVariance = variance;
        threshold = t;
      }
    }

    return threshold;
  }

  async processImage(input, outputDir = './output', method = 'average') {
    try {
      await this.ensureDirectoryExists(outputDir);

      let imageBuffer;
      let filename;

      if (typeof input === 'string' && input.startsWith('http')) {
        imageBuffer = await this.downloadImage(input);
        filename = path.basename(new URL(input).pathname);
      } else {
        imageBuffer = input;
        filename = 'local_image.jpg';
      }

      const image = sharp(imageBuffer);
      const { data, info } = await image
        .grayscale()
        .raw()
        .toBuffer({ resolveWithObject: true });

      // 计算直方图
      const histogram = new Array(256).fill(0);
      for (const pixel of data) {
        histogram[pixel]++;
      }

      let threshold;
      if (method === 'otsu') {
        threshold = this.calculateOtsuThreshold(histogram, info.width * info.height);
      } else {
        // 平均灰度法
        const sum = data.reduce((acc, val) => acc + val, 0);
        threshold = Math.round(sum / (info.width * info.height));
      }

      // 应用阈值进行二值化
      const binaryData = Buffer.from(data.map(pixel => pixel >= threshold ? 255 : 0));

      const fileExt = '.jpg';
      const baseName = path.parse(filename).name;
      const methodSuffix = method === 'otsu' ? '_otsu' : '_average';
      const outputFilename = `${baseName}${methodSuffix}${fileExt}`;
      const outputPath = path.join(outputDir, outputFilename);

      // 保存处理后的图片
      await sharp(binaryData, {
        raw: {
          width: info.width,
          height: info.height,
          channels: 1
        }
      })
        .jpeg()
        .toFile(outputPath);

      // 生成 TSPL 指令并保存
      const tsplCommands = this.tsplConverter.convertToTSPL(binaryData, info.width, info.height);
      const tsplOutputPath = path.join(outputDir, `${baseName}${methodSuffix}.tspl`);
      await this.tsplConverter.saveTSPLToFile(tsplCommands, tsplOutputPath);

      return {
        dimensions: { width: info.width, height: info.height },
        threshold,
        method,
        outputPath,
        tsplPath: tsplOutputPath
      };
    } catch (error) {
      throw new Error(`处理图片失败: ${error.message}`);
    }
  }
}