import fs from 'fs/promises';

export class TSPLConverter {
    constructor(options = {}) {
        this.options = {
            width: options.width || 384, // 打印宽度（点）
            density: options.density || 8, // 打印浓度 (0-15)
            speed: options.speed || 4,    // 打印速度 (1-4)
            ...options
        };
    }

    /**
     * 将二值化图像数据转换为 TSPL 指令
     * @param {Buffer} imageData - 二值化后的图像数据
     * @param {number} width - 图像宽度
     * @param {number} height - 图像高度
     * @returns {string} TSPL 指令
     */
    convertToTSPL(imageData, width, height) {
        // TSPL 指令初始化
        let tsplCommands = [];

        // 清除缓冲区
        tsplCommands.push('CLS');

        // 设置标签大小和间隙
        tsplCommands.push(`SIZE ${width} mm, ${height} mm`);
        tsplCommands.push('GAP 0 mm, 0 mm');

        // 设置打印浓度和速度
        tsplCommands.push(`DENSITY ${this.options.density}`);
        tsplCommands.push(`SPEED ${this.options.speed}`);

        // 将图像数据转换为十六进制字符串
        const hexData = this.convertImageDataToHex(imageData, width, height);

        // BITMAP 命令格式：x, y, width, height, mode, binary data
        tsplCommands.push(`BITMAP 0, 0, ${width}, ${height}, 0, ${hexData}`);

        // 打印命令
        tsplCommands.push('PRINT 1');

        return tsplCommands.join('\n');
    }

    /**
     * 将图像数据转换为十六进制格式
     */
    convertImageDataToHex(imageData, width, height) {
        let hexString = '';
        for (let i = 0; i < imageData.length; i++) {
            const hex = (imageData[i] === 0 ? 1 : 0).toString(16);
            hexString += hex.padStart(2, '0');
        }
        return hexString;
    }

    /**
     * 保存 TSPL 指令到文件
     */
    async saveTSPLToFile(tsplCommands, outputPath) {
        try {
            await fs.writeFile(outputPath, tsplCommands, 'utf8');
            return outputPath;
        } catch (error) {
            throw new Error(`保存 TSPL 文件失败: ${error.message}`);
        }
    }
}